@@ -38,6 +38,71 @@ class WorkflowTests(unittest.TestCase):
         sm._transition_order = ['publish', 'reject', 'retract', 'submit']
         return sm
 
+    def _makePopulatedOverlappingTransitions(
+        self, state_callback=None, transition_callback=None,
+        permission_checker=None):
+        sm = self._makePopulated(state_callback, transition_callback)
+        sm.permission_checker = permission_checker
+
+        sm._transition_data['submit2'] = dict(
+            name='submit2',
+            from_state='private',
+            to_state='pending',
+            callback=transition_callback,
+            )
+        sm._transition_order.append('submit2')
+        return sm
+
+    def test_transition_to_state_two_transitions_second_works(self):
+        args = []
+        def dummy(content, info):
+            args.append((content, info))
+
+        def checker(permission, context, request):
+            return permission == 'allowed'
+
+        sm = self._makePopulatedOverlappingTransitions(
+            transition_callback=dummy,
+            permission_checker=checker,
+            )
+
+        sm._transition_data['submit']['permission'] = 'forbidden'
+        sm._transition_data['submit2']['permission'] = 'allowed'
+
+        ob = DummyContent()
+        ob.state = 'private'
+        sm.transition_to_state(ob, object(), 'pending')
+        self.assertEqual(len(args), 1)
+        self.assertEqual(args[0][1].transition['name'], 'submit2')
+
+    def test_transition_to_state_two_transitions_none_works(self):
+        callback_args = []
+        def dummy(content, info):
+            callback_args.append((content, info))
+
+        checker_args = []
+        def checker(permission, context, request):
+            checker_args.append((permission, context, request))
+            return permission == 'allowed'
+
+        sm = self._makePopulatedOverlappingTransitions(
+            transition_callback=dummy,
+            permission_checker=checker,
+            )
+
+        sm._transition_data['submit']['permission'] = 'forbidden1'
+        sm._transition_data['submit2']['permission'] = 'forbidden2'
+
+        ob = DummyContent()
+        ob.state = 'private'
+        from repoze.workflow import WorkflowError
+        self.assertRaises(WorkflowError, sm.transition_to_state,
+                          ob, object(), 'pending')
+        self.assertEqual(len(callback_args), 0)
+        self.assertEqual(len(checker_args), 2)
+        self.assertEqual([a[0] for a in checker_args],
+                         ['forbidden1', 'forbidden2'])
+
     def test_class_conforms_to_IWorkflow(self):
         from zope.interface.verify import verifyClass
         from repoze.workflow.interfaces import IWorkflow
@@ -618,10 +683,7 @@ class WorkflowTests(unittest.TestCase):
         self.assertEqual(args, [('view', None, request)])
 
     def test_transition_request_is_None(self):
-        args = []
-        def checker(*arg):
-            args.append(arg)
-            return False
+        def checker(*arg): raise NotImplementedError
         workflow = self._makeOne(permission_checker=checker)
         transitioned = []
         def append(content, name, context=None, guards=(), skip_same=True):
@@ -641,13 +703,9 @@ class WorkflowTests(unittest.TestCase):
         permitted = transitioned['guards'][0]
         info = DummyCallbackInfo(transition = {'permission':'view'})
         self.assertEqual(None, permitted(None, info))
-        self.assertEqual(args, []) # not called
 
     def test_transition_permission_is_None(self):
-        args = []
-        def checker(*arg):
-            args.append(arg)
-            return False
+        def checker(*arg): raise NotImplementedError
         workflow = self._makeOne(permission_checker=checker)
         transitioned = []
         def append(content, name, context=None, guards=()):
@@ -667,7 +725,6 @@ class WorkflowTests(unittest.TestCase):
         permitted = transitioned['guards'][0]
         info = DummyCallbackInfo(transition = {})
         self.assertEqual(None, permitted(None, info))
-        self.assertEqual(args, []) # not called
 
     def test_transition_to_state_permissive(self):
         args = []
@@ -725,10 +782,7 @@ class WorkflowTests(unittest.TestCase):
         self.assertEqual(args, [('view', None, request)])
 
     def test_transition_to_state_request_is_None(self):
-        args = []
-        def checker(*arg):
-            args.append(arg)
-            return False
+        def checker(*arg): raise NotImplementedError
         workflow = self._makeOne(permission_checker=checker)
         transitioned = []
         def append(content, name, context=None, guards=(), skip_same=True):
@@ -748,13 +802,9 @@ class WorkflowTests(unittest.TestCase):
         permitted = transitioned['guards'][0]
         info = DummyCallbackInfo(transition = {'permission':'view'})
         self.assertEqual(None, permitted(None, info))
-        self.assertEqual(args, []) # not called
 
     def test_transition_to_state_permission_is_None(self):
-        args = []
-        def checker(*arg):
-            args.append(arg)
-            return False
+        def checker(*arg): raise NotImplementedError
         workflow = self._makeOne(permission_checker=checker)
         transitioned = []
         def append(content, name, context=None, guards=(), skip_same=True):
@@ -775,7 +825,6 @@ class WorkflowTests(unittest.TestCase):
         permitted = transitioned['guards'][0]
         info = DummyCallbackInfo(transition = {})
         self.assertEqual(None, permitted(None, info))
-        self.assertEqual(args, []) # not called
 
     def test_get_transitions_permissive(self):
         args = []
@@ -963,12 +1012,6 @@ class TestProcessWFList(unittest.TestCase):
         from repoze.workflow.workflow import process_wf_list
         return process_wf_list(wf_list, context)
 
-    def _getIContent(self):
-        from zope.interface import Interface
-        class IContent(Interface):
-            pass
-        return IContent
-
     def test_nothing_in_wf_list_returns_None(self):
         result = self._callFUT([], None)
         self.assertEqual(result, None)
@@ -981,8 +1024,7 @@ class TestProcessWFList(unittest.TestCase):
 
     def test_context_is_None_elector_not_None_no_fallback(self):
         workflow = object()
-        def elector(context):
-            return False
+        def elector(context): return False
         wflist = [{'elector':elector, 'workflow':workflow}]
         result = self._callFUT(wflist, None)
         self.assertEqual(result, None)
@@ -990,8 +1032,7 @@ class TestProcessWFList(unittest.TestCase):
     def test_context_is_None_elector_not_None_with_fallback(self):
         workflow = object()
         default = object()
-        def elector(context):
-            return False
+        def elector(context): return False
         wflist = [{'elector':elector, 'workflow':workflow},
                   {'elector':None, 'workflow':default}]
         result = self._callFUT(wflist, None)
@@ -1002,7 +1043,6 @@ class TestProcessWFList(unittest.TestCase):
         def elector(context):
             return False
         wflist = [{'elector':elector, 'workflow':workflow}]
-        context = object()
         result = self._callFUT(wflist, object)
         self.assertEqual(result, None)
 
